<?php
/**
 * 文章版权插件,<a href="https://www.dbkuaizi.com/archives/75.html">使用帮助</a>
 *
 * @package Copyright
 * @author 两双筷子
 * @version 1.0.0
 * @link https://www.dbkuaizi.com
 */
class Copyright_Plugin implements Typecho_Plugin_Interface
{
    // 插件名称
    protected static $plugin_name = 'Copyright';

    // 插件激活
    public static function activate()
    {
        // 若没有版权字段则添加
        $db = Typecho_Db::get();
        $prefix = $db->getPrefix();
        // contents 表中若无 copyright 字段则添加
        if (!array_key_exists('copyright', $db->fetchRow($db->select()->from('table.contents'))))
            $db->query("ALTER TABLE `{$prefix}contents` ADD `copyright` varchar(20) DEFAULT ''");

       // 挂载插件
        Typecho_Plugin::factory('admin/write-post.php')->option = array('Copyright_Plugin', 'render');
        Typecho_Plugin::factory('Widget_Contents_Post_Edit')->finishPublish = array(__CLASS__, 'change');
        // 返回提示信息
        return '插件安装成功';
    }

    // 禁用
    public static function deactivate()
    {
        $delFields = Typecho_Widget::widget('Widget_Options')->plugin(self::$plugin_name)->del_fields;
        $delstr = '';
        if($delFields){
            $db = Typecho_Db::get();
            $prefix = $db->getPrefix();
            $db->query("ALTER TABLE `{$prefix}contents` DROP `copyright`;");
            $delstr = '，所设文章版权已清除';
        }

        return '插件卸载成功' . $delstr;
    }

    // 插件配置面板
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        $cp_json_desc = '需是完整的JSON格式：<code>{"版权类别1":"版权描述1","版权类别2":"版权描述2"}</code>，当版权描述为 “off” 时则不会显示版权';
        $cp_json_cont = <<<EOT
{
    "CC版权": "本篇博文采用<a href='https://creativecommons.org/licenses/by-nc-nd/4.0/legalcode.zh-Hans'>《CC 协议》</a>，转载必须注明作者和本文链接",
    "禁止转载": "本篇博文未经作者同意禁止转载",
    "自由转载": "本篇博文可自由转载，无需作者同意且无需授权",
    "不显示": "off"
}
EOT;
        $cp_json = new Typecho_Widget_Helper_Form_Element_Textarea('cp_json', NULL, $cp_json_cont, _t('版权配置'),$cp_json_desc);
        $form->addInput($cp_json);

        $cp_tpl_desc = "版权显示模板，可以写HTML/CSS，模板分类变量：<code>{{type}}</code>，版权正文变量：<code>{{content}}</code>";
        $cp_tpl_cont = <<<EOT
<blockquote>
   <strong>{{type}}</strong>： {{content}}
</blockquote>
EOT;
        $cp_tpl = new Typecho_Widget_Helper_Form_Element_Textarea('cp_tpl', NULL, $cp_tpl_cont, _t('版权显示模板'),$cp_tpl_desc);
        $form->addInput($cp_tpl);

        // 无版权缺省
        $cp_default = new Typecho_Widget_Helper_Form_Element_Radio('cp_default',
            array(1=>_t('显示第一条'),0=>_t('不显示')), '1', _t('缺省显示'),_t('文章未选择版权或所设置版权不存在时的处理方式'));
        $form->addInput($cp_default);
        $del_fields = new Typecho_Widget_Helper_Form_Element_Radio('del_fields',
            array(0=>_t('保留数据'),1=>_t('删除数据'),), '0', _t('卸载设置'),_t('卸载插件时已设置版权数据是否保留'));
        $form->addInput($del_fields);

    }

    // 个人用户的配置面板
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}

    public static function render($post)
    {
        $copyright_arr = json_decode(Typecho_Widget::widget('Widget_Options')->plugin(self::$plugin_name)->cp_json,true);
        $optoin = '';

        $db = Typecho_Db::get();
        $row = $db->fetchRow($db->select('copyright')->from('table.contents')->where('cid = ?', $post->cid));

        // 判断 版权json 是否合法
        if (is_array($copyright_arr)) {

            foreach ($copyright_arr as $key => $val) {
                // 判断 当前选项 是否是已选择的
                if($key == $row['copyright']) {
                    $optoin .= "<option value='{$key}' selected>{$key}</option>" . PHP_EOL;
                } else {
                    $optoin .= "<option value='{$key}'>{$key}</option>" . PHP_EOL;
                }

            }
            $str = <<<EOT
 <section class="typecho-post-option visibility-option">
    <label for="Copyright" class="typecho-label">版权</label>
    <p>
        <select name="copyright" style="width: 100%;">
            {$optoin}
        </select>
    </p>
</section>
EOT;

        } else {
            $str = <<<EOT
 <section class="typecho-post-option visibility-option">
    <label for="Copyright" class="typecho-label">版权</label>
    <p style="color: red">
        版权配置JSON格式不正确
    </p>
</section>
EOT;
        }


        echo $str;
    }

    // 修改文章时触发
    public static function change($contents, $post)
    {
        $copyright = $post->request->get('copyright','default');
        $db = Typecho_Db::get();
        $sql = $db->update('table.contents')->rows(array('copyright' => $copyright))->where('cid = ?', $post->cid);
        $db->query($sql);
    }

    /**
     * 输出版权信息
     */
    public static function Output($obj)
    {
        // 获取所有启用插件数组，判断当前插件是否被启用
        $plugin_arr = Typecho_Plugin::export();
        // 如果插件未启用 则直接返回
        if (!array_key_exists(self::$plugin_name,$plugin_arr['activated'])) {
            return;
        }

        // 获取配置
        $cp_json = json_decode(Typecho_Widget::widget('Widget_Options')->plugin(self::$plugin_name)->cp_json,true);
        $cp_tpl = Typecho_Widget::widget('Widget_Options')->plugin(self::$plugin_name)->cp_tpl;
        $cp_default = Typecho_Widget::widget('Widget_Options')->plugin(self::$plugin_name)->cp_default;

        // 如果所选不存在，并且缺省不显示 则返回
        if (empty($cp_json[$obj->copyright]) && empty($cp_default)) {
            return;
        }

        // 如果当前选项的value 是off，则不显示版权
        if (!empty($cp_json[$obj->copyright]) && in_array($cp_json[$obj->copyright], array('off',"OFF"))) {
            return;
        }

        $replace_arr = array("{{type}}","{{content}}"); // 替换的内容

        // 如果选择不存在，就选中第一个
        if (empty($cp_json[$obj->copyright])) {
            $key1 = key($cp_json);
            $res_arr = array($key1, $cp_json[$key1]); // 被替换
        } else {
            $res_arr = array($obj->copyright, $cp_json[$obj->copyright]); // 被替换
        }
        // 输出版权
        echo str_replace($replace_arr,$res_arr,$cp_tpl);
    }
}